/*
    MPEG Maaate: An Australian MPEG audio analysis toolkit
    Copyright (C) 2000 Commonwealth Scientific and Industrial Research Organisation
    (CSIRO), Australia.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#ifndef DECODER_H
#define DECODER_H

#include "MPEGfile.H"

typedef enum { LEFT=0, RIGHT, STEREO } Channels;

#ifdef __cplusplus

#include <algorithm>
#include <cstdio>
#include <cmath>
#include <iostream>
#include <string>

using namespace std;


//--------------------------------------------------------------------------

// Class to access decoded pcm samples of an MPEG audio file by
// using MPEG Maaate Parser (MaaateP).
// The user has to provide the buffer for the decoder to write
// the PCM data into. It has to be an array of shorts and the
// result will be a collection of 16 bit linear PCM samples,
// left/right stored interleaved if stereo, in the byteorder native
// to the machine it runs on.
class MDecoder {

public:

  // constructor for an exisiting MPEGfile object
  MDecoder (MPEGfile *mpf) : frame(0), granule(0), subband(0), bandsample(0), mp(mpf), underControl(false) {}

  // constructor for a named mpeg file
  MDecoder (char *mpfile) : frame(0), granule(0), subband(0), bandsample(0), underControl(true) {mp = new MPEGfile (string(mpfile));}

  // destructor
  ~MDecoder() {if (underControl) delete mp;}

  bool   set_sample_offset (long frame);
  bool   set_sample_offset (float time);
  long   get_sample_offset_frame ();
  float  get_sample_offset_time ();
  double samplingrate ();
  int    channels ();

  // returns number of bytes in buffer
  long  decode_frame (long frameno, short *buffer, Channels ch);
  
  long  decode (long frame, short *buffer, long *nr_samples, Channels ch);
  float decode (float time, short *buffer, long *nr_samples, Channels ch);

private:

  MPEGfile *mp;
  char     *mpfile;
  bool      underControl;

  long      frame;
  int       granule;
  int       subband;
  int       bandsample;

};

//--------------------------------------------------------------------------

// inline operators

extern "C" {

#else /* __cplusplus */

typedef struct MDecoder MDecoder;

#endif /* __cplusplus */


  MDecoder * maaateP_new_MDecoder_from_mp (MPEGfile *mp);
  MDecoder * maaateP_new_MDecoder_from_file (char *mpfile);
  void       maaateP_MDecoder_destroy (MDecoder *d);

  int    maaateP_MDecoder_set_sample_offset_frame (MDecoder * d, long frame);
  int    maaateP_MDecoder_set_sample_offset_time  (MDecoder * d, float time);
  long   maaateP_MDecoder_get_sample_offset_frame (MDecoder * d);
  float  maaateP_MDecoder_get_sample_offset_time  (MDecoder * d);
  double maaateP_MDecoder_samplingrate (MDecoder * d);
  int    maaateP_MDecoder_channels     (MDecoder * d);

  long   maaateP_MDecoder_decode_frame (MDecoder * d, long frameno,
					short *buffer, Channels ch);

  long   maaateP_MDecoder_decode_from_frame (MDecoder * d,
					     long frame,
					     short *buffer,
					     long *nr_samples,
					     Channels ch);

  float  maaateP_MDecoder_decode_from_time  (MDecoder * d,
					     float time,
					     short *buffer,
					     long *nr_samples,
					     Channels ch);


#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
