/*
    MPEG Maaate: An Australian MPEG audio analysis toolkit
    Copyright (C) 2000 Commonwealth Scientific and Industrial Research Organisation
    (CSIRO), Australia.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef LAYER1_H
#define LAYER1_H

#include "allLayers.H"
#include "MPEGfile.H"

using namespace std;

//--------------------------------------------------------------------------

// class definition
class Layer1 : public AllLayers {
public:
    Layer1(MPEGfile *mp);


    inline int    bitallocation (unsigned int ch, 
				 unsigned int sb);
    inline int    scfsi (unsigned int ch, unsigned int sb) {};
    inline float  scalefactor (unsigned int ch, 
			       unsigned int sb, 
			       unsigned int ss=0);
    inline int    sample (unsigned int ch, 
			  unsigned int sb, 
			  unsigned int no, 
			  unsigned int ss=0);
    inline double restored_sample (unsigned int ch, 
				   unsigned int sb, 
				   unsigned int no, 
				   unsigned int ss=0);
    inline short  pcm_sample (unsigned int ch, 
			      unsigned int sb, 
			      unsigned int no, 
			      unsigned int ss=0);
    
    unsigned int calc_CRC();

    bool parse_data(bool decode);

private:
    MPEGfile *frame;

    // AUDIO DATA CONTENT:
    // channels()*subbands() bitallocations (4 bit to be read per entry)
    unsigned int allocation[2][SBLIMIT];
    // channels()*subbands() scale factor indexes (6 bit to be read per entry)
    unsigned int scale_index[2][SBLIMIT];
    // 12*channels()*subbands() sample data (2-15 bit to be read per entry)
    unsigned int samples[12][2][SBLIMIT];
    // 12*channels()*subbands() restored samples
    double restoredsamples[12][2][SBLIMIT];
    // 12*channels()*subbands() restored pcm samples
    short pcm[12][2][SBLIMIT];

    void decode_bitalloc();
    void decode_scale();
    void buffer_samples();
    void restore_samples();

    inline unsigned int bits_per_sample (unsigned int ch, unsigned int sb);
};

//--------------------------------------------------------------------------

// inline operators

int 
Layer1::bitallocation (unsigned int channel, unsigned int subband) 
{
    return allocation[channel][subband];
}

unsigned int 
Layer1::bits_per_sample (unsigned int channel, unsigned int subband) 
{
    if (allocation[channel][subband]==0 || allocation[channel][subband]==15)
	return 0;
    else
	return (allocation[channel][subband] + 1);
}

float
Layer1::scalefactor (unsigned int channel, unsigned int subband, unsigned int ss=0) 
{
    return maaate_scalefactors[scale_index[channel][subband]];
}

int
Layer1::sample (unsigned int channel, unsigned int subb, unsigned int number, unsigned int ss=0) 
{
    return samples[number][channel][subb];
}

double
Layer1::restored_sample (unsigned int ch, unsigned int subb, unsigned int no, unsigned int ss=0) 
{
    return restoredsamples[no][ch][subb];
}

short
Layer1::pcm_sample (unsigned int ch, unsigned int subb, unsigned int no, unsigned int ss=0) 
{
    return pcm[no][ch][subb];
}

#endif
