/*
    MPEG Maaate: An Australian MPEG audio analysis toolkit
    Copyright (C) 2000 Commonwealth Scientific and Industrial Research Organisation
    (CSIRO), Australia.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef LAYER2_H
#define LAYER2_H

#include "allLayers.H"
#include "MPEGfile.H"

using namespace std;

//--------------------------------------------------------------------------

// class definition
class Layer2 : public AllLayers {
public:
    Layer2 (MPEGfile *mp);
 
    inline int    bitallocation (unsigned int ch, unsigned int sb);
    inline int    scfsi (unsigned int ch, 
			 unsigned int sb);
    inline float  scalefactor (unsigned int ch, 
			       unsigned int sb, 
			       unsigned int no);
    inline int    sample (unsigned int ch,
			  unsigned int sb,
			  unsigned int no,
			  unsigned int ss);
    inline double restored_sample (unsigned int ch, 
				   unsigned int sb, 
				   unsigned int no,
				   unsigned int ss);
    inline short  pcm_sample (unsigned int ch, 
			      unsigned int sb, 
			      unsigned int no,
			      unsigned int ss);

    unsigned int calc_CRC();

    bool parse_data(bool decode);

private:
    MPEGfile *frame;

    // AUDIO DATA CONTENT:
    // channels()*subbands() bitallocations (2-4 bit to be read per entry)
    unsigned int allocation[2][SBLIMIT];
    // channels()*subbands() scale factor selections (2 bit per entry)
    unsigned int scfsis[2][SBLIMIT];
    // channels()*3*subbands() scale factor indexes (6 bit per entry)
    unsigned int scale_index[2][3][SBLIMIT];
    // 3*12*channels()*subbands() sample data (3-16 bit to be read per entry)
    unsigned int samples[12][2][3][SBLIMIT];

    // 12*channels()*subbands() restored samples
    double restoredsamples[12][2][3][SBLIMIT];
    // 12*channels()*subbands() restored samples
    short pcm[12][2][3][SBLIMIT];

    // subband allocation table structure for layer II
    struct SubbandTable {
      unsigned int    steps;
      unsigned int    bits;
      unsigned int    group;
      unsigned int    quant;
    };

    int table;
    SubbandTable alloctable[SBLIMIT][16];
    
    void pick_table();
    void read_bit_alloc();
    
    void decode_bitalloc();
    void decode_scale();
    void buffer_samples();
    void restore_samples();

    inline unsigned int grouping (unsigned int ch, unsigned int sb);
    inline unsigned int bits_per_sample (unsigned int ch, unsigned int sb);
    inline unsigned int bits_per_samplecode (unsigned int ch, unsigned int sb);

};

//--------------------------------------------------------------------------

// inline operators

int 
Layer2::bitallocation (unsigned int channel, unsigned int subband) 
{
    return allocation[channel][subband];
}
 
int 
Layer2::scfsi (unsigned int channel, unsigned int subband) 
{
    return scfsis[channel][subband];
}

unsigned int
Layer2::grouping (unsigned int channel, unsigned int subband) 
{
    return alloctable[subband][allocation[channel][subband]].group;
}

unsigned int 
Layer2::bits_per_sample (unsigned int channel, unsigned int subband) 
{
    return alloctable[subband][allocation[channel][subband]].bits;
}

unsigned int 
Layer2::bits_per_samplecode (unsigned int channel, unsigned int subband) 
{
    return alloctable[subband][allocation[channel][subband]].steps;
}

float
Layer2::scalefactor (unsigned int channel, unsigned int subband, unsigned int no) 
{
    if (subband >= frame->subbands()) {
	return 0;
    } else {
	return maaate_scalefactors[scale_index[channel][no][subband]];
    }
}
 
int
Layer2::sample (unsigned int channel, unsigned int subb, unsigned int number, unsigned int ss) 
{
    return samples[number][channel][ss][subb];
}
 
double
Layer2::restored_sample (unsigned int ch, unsigned int subb, unsigned int no, unsigned int ss) 
{
    return restoredsamples[no][ch][ss][subb];
}

short
Layer2::pcm_sample (unsigned int channel, unsigned int subb, unsigned int number, unsigned int ss) 
{
    return pcm[number][channel][ss][subb];
}
#endif
