/*
    MPEG Maaate: An Australian MPEG audio analysis toolkit
    Copyright (C) 2000 Commonwealth Scientific and Industrial Research Organisation
    (CSIRO), Australia.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef LAYER3_H
#define LAYER3_H

#include "allLayers.H"
#include "MPEGfile.H"

using namespace std;

//--------------------------------------------------------------------------

// class definition
class Layer3 : public AllLayers {

  // Layer 3 specific header
  struct Sideinfo {
    unsigned int main_data_begin;  // 9 bit
    unsigned int private_bits;     // 3/5 bit
    struct {
      unsigned scfsi[4];         // 1 bit
      struct gr_info_s {
	unsigned part2_3_length;         // 12 bit
	unsigned big_values;             // 9 bit
	unsigned global_gain;            // 8 bit
	unsigned scalefac_compress;      // 4 bit
	unsigned window_switching_flag;  // 1 bit
	unsigned block_type;             // 2 bit
	unsigned mixed_block_flag;       // 1 bit
	unsigned table_select[3];        // 5 bit
	unsigned subblock_gain[3];       // 3 bit
	unsigned region0_count;          // 4 bit
	unsigned region1_count;          // 3 bit
	unsigned preflag;                // 1 bit
	unsigned scalefac_scale;         // 1 bit
	unsigned count1table_select;     // 1 bit
      } gr[2]; // for each granule
    } ch[2]; // for each channel
  };
 
  // structure to store scalefactors in
  struct ScaleFacTable {
    int l[22];            // sfb (long windows)
    int s[3][13];         // window*sfb (short windows)
  };
 
public:
    Layer3(MPEGfile *mp);
    
    inline int    bitallocation (unsigned int ch, unsigned int sb) {};
    inline int    scfsi (unsigned int ch, unsigned int scfclass);
           float  scalefactor (unsigned int ch, unsigned int ss, 
			       unsigned int gr);
    inline int    sample (unsigned int ch, unsigned int gr, unsigned int sb, 
			  unsigned int ss);
    inline double restored_sample (unsigned int ch, unsigned int gr, 
				   unsigned int sb, unsigned int ss);
    inline short  pcm_sample (unsigned int ch, unsigned int gr, 
			      unsigned int sb, unsigned int ss);

    // more audio data information
    inline unsigned int part2_3_length(unsigned int ch, unsigned int gr);
    inline unsigned int big_values (unsigned int ch, unsigned int gr);
    inline unsigned int count1_values (unsigned int ch, unsigned int gr);
    inline unsigned int global_gain (unsigned int ch, unsigned int gr);
    inline unsigned int scalefac_compress (unsigned int ch, unsigned int gr);
    inline bool         window_switching (unsigned int ch, unsigned int gr);
    inline unsigned int blocktype (unsigned int ch, unsigned int gr);
    inline const char*  blocktype_str (unsigned int ch, unsigned int gr);
    inline bool         mixedblock (unsigned int ch, unsigned int gr);
    inline unsigned int table_select(unsigned int ch, unsigned int gr, 
				     unsigned int area);
    inline unsigned int subblock_gain(unsigned int ch, unsigned int gr, 
				      unsigned int area);
    inline unsigned int region0_samps (unsigned int ch, unsigned int gr);
    inline unsigned int region1_samps (unsigned int ch, unsigned int gr);
    inline unsigned int region2_samps (unsigned int ch, unsigned int gr);
    inline unsigned int preflag (unsigned int ch, unsigned int gr);
    inline unsigned int scalefac_scale (unsigned int ch, unsigned int gr);
    inline unsigned int count1table_select(unsigned int ch, unsigned int gr);
    inline unsigned int scf_band_bound_l (unsigned int sbindex);
    inline unsigned int scf_band_bound_s(unsigned int sbindex);
    unsigned int slen1 (unsigned int ch, unsigned int gr);
    unsigned int slen2 (unsigned int ch, unsigned int gr);

    unsigned int calc_CRC();

    bool parse_data(bool decode);

    void printSideinfo();

    bool clearinterbuffer();

private:
    MPEGfile *frame;

#ifdef DEBUG
    long noReadBits;
#endif

    // audio data stored in intermediate bitbuffer.
    // This is a "circular buffer" and takes any spare bytes from
    // the frame's buffer to store between indexes bufstart and bufend.
#define MAX_INTER_BUFSIZE 1920 // max no of bytes stored (7680 bit/granule)
    unsigned char inter_buffer[MAX_INTER_BUFSIZE];
    unsigned int bufstart, bufend;
    unsigned int whichbit;
    unsigned int main_data_bitsread;
    unsigned int endSpectralBits;   // to remember how many bits to read
    unsigned int spectral_bitsread; // to calculate count1 values to read
    unsigned int readbits(unsigned int nobits);
    bool         setbufstart();
    bool         savetointerbuffer();

    // read huffman decoder table
    bool read_decoder_table();

    // remember how many count1-values were processed
    // granules()*channels() count1 values
    int count1_vals[2][2];

    // AUDIO DATA CONTENT:
    // 9, 17 or 32 byte to be read into sideinfo header
    Sideinfo si;
    // 0-4 bit read per entry
    // granules()*channels() scalefactors
    ScaleFacTable scalefac[2][2];
    // quantized value for frequency lines (0-19 bit to be read per entry)
    // granules()*channels()*subbands()(32)*scalefactorbands()(18)
    long int is[2][2][SBLIMIT][SSLIMIT];
    // granules()*channels()*subbands()*scalefactorbands() restored samples
    double xr[2][2][SBLIMIT][SSLIMIT];
    // granules()*channels()*subbands()*scalefactorbands() restored stereo
    double lr[2][2][SBLIMIT][SSLIMIT];
    // granules()*channels()*subbands()*scalefactorbands() reordered samples
    double ro[2][2][SBLIMIT][SSLIMIT];
    // granules()*channels()*subbands()*scalefactorbands() hybrid filter input
    double hybridIn[2][2][SBLIMIT][SSLIMIT];
    // granules()*channels()*scalefactorbands()*subbands() hybrid filter output
    double hybridOut[2][2][SSLIMIT][SBLIMIT];
    // granules()*channels()*subsubbands()*subbands() pcm samples
    short pcm[2][2][SSLIMIT][SBLIMIT];

    // frame processing functions
    void decode_sideinfo();
    void decode_scale_I(int gr, int ch);
    void decode_scale_II(int gr, int ch); // for MPEG2
    void decode_huffmanbits(int gr, int ch); // sample data
    bool huffman_decoder(struct huffcodetab *h, // huffman code record
			 unsigned int *x,       // decoded x value
			 unsigned int *y,       // decoded y value
			 unsigned int *v,       // decoded v value
			 unsigned int *w);
    void restore_samples(int gr, int ch);
    void decode_jstereo(int gr);
    void i_stereo_k_values(int ch, int gr, char is_pos, int i, 
			   double k[2][SBSSLIMIT]); // MPEG2
    void reorder(int gr, int ch);
    void antialias(int gr, int ch);
    void hybrid(int gr, int ch);
    void inv_mdct(double in[18], double out[36], int bt);
    void polyphase(int ch, int gr);

    // access to class member values
    inline unsigned int main_data_begin();
    inline unsigned int private_bits();
    inline unsigned int region0_count (unsigned int ch, unsigned int gr);
    inline unsigned int region1_count (unsigned int ch, unsigned int gr);
           unsigned int scfsi_group (unsigned int ch, unsigned int group);
    inline unsigned int sideinfo_length();
    inline unsigned int granules();
    inline unsigned int main_data_slots();

    unsigned int scf_bands (unsigned int ch, unsigned int gr);
};

//--------------------------------------------------------------------------

// inline operators

// returns scfsi for channel and scalefactor class
// this tells whether the scalefactors for granule 0 are also used for gr 1
int 
Layer3::scfsi (unsigned int channel, unsigned int scfclass)
{
    return si.ch[channel].scfsi[scfclass];
}

// returns the samples of the 576 frequency lines (decoded huffman data)
int
Layer3::sample (unsigned int ch, 
		unsigned int gr, 
		unsigned int sb, 
		unsigned int ss)
{
    return is[gr][ch][sb][ss];
}

// returns the decoded pcm samples
double
Layer3::restored_sample (unsigned int channel, unsigned int granule, 
			 unsigned int subband, unsigned int subsubband)
{
    return ro[granule][channel][subband][subsubband];
    //    return hybridIn[granule][channel][subband][subsubband];
}

// returns the decoded pcm samples
short
Layer3::pcm_sample (unsigned int channel, unsigned int granule, 
		    unsigned int subband, unsigned int subsubband)
{
    return pcm[granule][channel][subsubband][subband];
}

// return main_data_begin offset
unsigned int
Layer3::main_data_begin()
{
    return si.main_data_begin;
}

// return private bits
unsigned int
Layer3::private_bits()
{
    return si.private_bits;
}

// return length of part2 & 3 (scalefactors+huffman data)
unsigned int
Layer3::part2_3_length(unsigned int channel, unsigned int granule)
{
    return si.ch[channel].gr[granule].part2_3_length;
}

// big values region
// contains no of sample pairs encoded from lower part of spectrum
unsigned int
Layer3::big_values(unsigned int channel, unsigned int granule)
{
    return si.ch[channel].gr[granule].big_values;
}

// count1 values region
// contains no of sample quadruples encoded from middle part of spectrum
unsigned int
Layer3::count1_values(unsigned int channel, unsigned int granule)
{
    return count1_vals[granule][channel];
}

// return global gain (= quantizer step size)
unsigned int
Layer3::global_gain(unsigned int channel, unsigned int granule)
{
    return si.ch[channel].gr[granule].global_gain;
}

// return scalefac_compress which encodes biallocation for scalefactors
unsigned int
Layer3::scalefac_compress(unsigned int channel, unsigned int granule)
{
    return si.ch[channel].gr[granule].scalefac_compress;
}

// returns 0 for long windows and 1 for transition, short or mixed windows
bool
Layer3::window_switching(unsigned int channel, unsigned int granule)
{
    return (bool) si.ch[channel].gr[granule].window_switching_flag;
}

// returns 1 for start of short windows, 2 for short windows, 3 for end of
// short windows; 0 otherwise
unsigned int
Layer3::blocktype(unsigned int channel, unsigned int granule)
{
    return si.ch[channel].gr[granule].block_type;
}

// returns string for description of block type
const char*
Layer3::blocktype_str(unsigned int channel, unsigned int granule)
{
    if (si.ch[channel].gr[granule].window_switching_flag) {
	// window switching
	switch (si.ch[channel].gr[granule].block_type) {
	case 1:
	    if (mixedblock(channel,granule)) {
		return "start block (long block), mixed";
	    } else {
		return "start block (long block)";
	    }
	case 2:
	    if (mixedblock(channel,granule)) {
		return "short blocks, mixed";
	    } else {
		return "short blocks";
	    }
	case 3:
	    if (mixedblock(channel,granule)) {
		return "stop block (long block), mixed";
	    } else {
		return "stop block (long block)";
	    }
	default:
	    return "reserved";
	}
    } else {
	return "long block";
    }
}

// returns true if encoded block is mixed (i.e. two lowest subbands are
// transformed with long windows, upper 30 subbands with short windows);
// false otherwise
bool
Layer3::mixedblock(unsigned int channel, unsigned int granule)
{
    return (bool) si.ch[channel].gr[granule].mixed_block_flag;
}

// returns which huffman table to select for big_values area
unsigned int
Layer3::table_select(unsigned int channel, unsigned int granule, unsigned int area)
{
    return si.ch[channel].gr[granule].table_select[area];
}

// returns gain of area
unsigned int
Layer3::subblock_gain(unsigned int channel, unsigned int granule, unsigned int area)
{
    return si.ch[channel].gr[granule].subblock_gain[area];
}

// returns region 0 boundary
unsigned int
Layer3::region0_count(unsigned int channel, unsigned int granule)
{
    return si.ch[channel].gr[granule].region0_count;
}

// returns region 1 boundary
unsigned int
Layer3::region1_count(unsigned int channel, unsigned int granule)
{
    return si.ch[channel].gr[granule].region1_count;
}

// returns no of samples in region0
unsigned int
Layer3::region0_samps(unsigned int channel, unsigned int granule)
{
    if (big_values(channel,granule) > 0) {
	if (window_switching(channel,granule) && 
	    blocktype(channel,granule)==2) {
	    // pure short windows
	    return 36; // as scf_band_bound_s[9/3]=12 for all sampling rates
	} else {
	    return min (scf_band_bound_l (region0_count(channel,granule)+1),
			big_values(channel,granule)*2);
	}
    } else {
	return 0;
    }
}

// returns no of samples in region1
unsigned int
Layer3::region1_samps(unsigned int channel, unsigned int granule)
{
    if (big_values(channel,granule) > 0) {
	if (window_switching(channel,granule) && 
	    blocktype(channel,granule)==2) {
	    // pure short windows, no region 2
	    return (2*big_values(channel,granule) - 36);
	} else {
	    return (min (scf_band_bound_l (region0_count(channel,granule) +
					   region1_count(channel,granule) + 2),
			 big_values(channel,granule)*2) -
			region0_samps(channel,granule));
	}
    } else {
	return 0;
    }
}

// returns no of samples in region2
unsigned int
Layer3::region2_samps(unsigned int channel, unsigned int granule)
{
    if (big_values(channel,granule) > 0) {
	if (window_switching(channel,granule) && 
	    blocktype(channel,granule)==2) {
	    // pure short windows, no region 2
	    return 0;
	} else {
	    unsigned int region0_1_samps = region1_samps(channel,granule) +
		region0_samps(channel,granule);
	    if ((2*big_values(channel,granule)) > region0_1_samps) {
		return (2*big_values(channel,granule) - 
			region1_samps(channel,granule) -
			region0_samps(channel,granule));
	    } else {
		return 0;
	    }
	} 
    } else {
	return 0;
    }
}

// returns preemphasis flag
unsigned int
Layer3::preflag(unsigned int channel, unsigned int granule)
{
    return si.ch[channel].gr[granule].preflag;
}

// returns scalefactor scaling (i.e. quantized with stepsize of 2 or sqrt(2))
unsigned int
Layer3::scalefac_scale(unsigned int channel, unsigned int granule)
{
    return si.ch[channel].gr[granule].scalefac_scale;
}

// returns which huffman table to select for count1 area
unsigned int
Layer3::count1table_select(unsigned int channel, unsigned int granule)
{
    return si.ch[channel].gr[granule].count1table_select;
}

// returns the length of the layer3-specific header in bytes
unsigned int
Layer3::sideinfo_length()
{
    if (frame->version() == MPEG1) {
	if (frame->channels() == 1) {
	    return 17; // byte
	} else {
	    return 32; // byte
	}
    } else { // MPEG2
	if (frame->channels() == 1) {
	    return 9; // byte
	} else {
	    return 17; // byte
	}
    }	
}

// returns the no of granules
unsigned int
Layer3::granules()
{
  return ((frame->version() == MPEG2) ? 1 : 2);
}

// returns the no of slots of main_data
unsigned int 
Layer3::main_data_slots()
{
    int nSlots = (int) ((144 * frame->bitrate()) / frame->samplingrate());

    if (frame->version() == MPEG2) {
	nSlots /= 2;
    }
    nSlots -= sideinfo_length();
    if (frame->padding()) {
        nSlots++;
    }
    nSlots -= 4; // deduct header
    if (frame->crcprotected()) {
        nSlots -= 2; // deduct checksum
    }
 
    return nSlots;
}

#endif
