/*
    MPEG Maaate: An Australian MPEG audio analysis toolkit
    Copyright (C) 2000 Commonwealth Scientific and Industrial Research Organisation
    (CSIRO), Australia.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#ifndef PLUGINS_H
#define PLUGINS_H

// for checking files
#include <sys/types.h>
#include <sys/stat.h>

#include "module.H"

#ifdef __cplusplus

#include <string>
#include <list>
 
using namespace std;


//--------------------------------------------------------------------------

#define MAAATE_PLUGIN_API_MAJOR    1
#define MAAATE_PLUGIN_API_MINOR    0
#define MAAATE_PLUGIN_API_REVISION 0

// class to open one plugin library (just a support class for plugin class)
class PluginLibrary {
public:
  // open a library with dlopen and call its initialisation function, thus
  // loading and creating the list of contained modules
  PluginLibrary(string filename);

  // close a library with dlclose
  ~PluginLibrary();

  // Get the name of the library
  string filename () { return name; }

  // Get the list of modules accessible from this library this is kept
  // for information on the library's content.  The Plugins class then
  // contains a list of all the activated modules for an application.
  list<Module> * Modules () { return &modList; }

private:
  // keep name for Plugins class
  string name;

  // keep handle to opened library for destructor
  void * plib;

  // contains all modules accessible from this library,
  // created with initialisation function
  list<Module> modList;

};


// class to load all plugin libraries and 
// administrate a list of the modules contained therein
class Plugins {
public:
  // default constructor
  Plugins() { modList.clear(); };

  // Load every module that Maaate is aware of.
  void AddLibrariesMaaatePath ();

  // add statically linked modules to the module list
  void AddStaticModules();

  // functions to open libraries, thus creating PluginLibrary instances,
  // and appending to the class' module list
  //
  // - all modules contained in one library
  bool AddLibrary(string filename);
  // - all modules contained in all libraries in a directory
  void AddLibraries(string dirname);
  // - all modules contained in all libraries of all directories specified in a
  //   colon-separated path list
  void AddLibrariesPath(string pathlist);
  // - a module of a library (only if removed before)
  void AddModule(Module * module);

  // destructs a PluginLibrary and unloads all its modules
  void RemoveLibrary (string name);
  // removes one module from the module list
  // and adds it to the removed list
  void RemoveModule (Module * module);

  // return a list of available modules for an application
  list<Module> * Modules () { return &modList; }

  // get a specific module of a given name
  Module * GetModule (string name);

  // return a list of available modules for a library
  list<Module> * LibraryModules(string name);
  
private:
  // all available modules from all libraries
  list<Module> modList;

  list<Module> removedList;

};

extern "C" {
#else /* __cplusplus */
  typedef struct Plugins Plugins;
#endif

  Plugins * maaateA_new_plugins();

  void maaateA_add_libraries_maaate_path (Plugins * plugin);

  void maaateA_add_static_modules(Plugins * plugin);

  int  maaateA_add_library(Plugins * plugin, char * filename);

  void maaateA_add_libraries(Plugins * plugin, char * dirname);

  void maaateA_add_libraries_path(Plugins * plugin, char * pathlist);

  void maaateA_add_module(Plugins * plugin, Module * module);

  void maaateA_remove_library(Plugins * plugin, char * name);

  void maaateA_remove_module(Plugins * plugin, Module * module);

  ModuleList * maaateA_modules(Plugins * plugin);

  Module * maaateA_get_module(Plugins * plugin, char * name);

  ModuleList * maaateA_library_modules(Plugins * plugin, char * name);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
